<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Xclass\Frontend\Page;

/**
 * @package     imia_pageteaser
 * @subpackage  Xclass
 * @author      David Frerich <d.frerich@imia.de>
 */
class PageRepository extends \TYPO3\CMS\Frontend\Page\PageRepository
{
    /**
     * @var array
     */
    protected $externalDoktypes = array(103, 112, 113, 123, 133, 143);

    /**
     * init() MUST be run directly after creating a new template-object
     * This sets the internal variable $this->where_hid_del to the correct where clause for page records taking deleted/hidden/starttime/endtime/t3ver_state into account
     *
     * @param boolean $show_hidden If $show_hidden is TRUE, the hidden-field is ignored!! Normally this should be FALSE. Is used for previewing.
     * @return void
     * @see tslib_fe::fetch_the_id(), tx_tstemplateanalyzer::initialize_editor()
     * @todo Define visibility
     */
    public function init($show_hidden) {
        $this->where_groupAccess = '';
        $this->where_hid_del = ' AND pages.deleted=0 ';
        if (!$show_hidden) {
            $this->where_hid_del .= 'AND pages.hidden=0 ';
            $this->where_hid_del .= 'AND pages.starttime<=' . $GLOBALS['SIM_ACCESS_TIME'] . ' AND (pages.endtime=0 OR pages.endtime>' . $GLOBALS['SIM_ACCESS_TIME'] . ') ';
        }
        // Filter out new/deleted place-holder pages in case we are NOT in a versioning preview (that means we are online!)
        if (!$this->versioningPreview) {
            $this->where_hid_del .= ' AND NOT pages.t3ver_state>0';
        } else {
            // For version previewing, make sure that enable-fields are not de-selecting hidden
            // pages - we need versionOL() to unset them only if the overlay record instructs us to.
            // Copy where_hid_del to other variable (used in relation to versionOL())
            $this->versioningPreview_where_hid_del = $this->where_hid_del;
            // Clear where_hid_del
            $this->where_hid_del = ' AND pages.deleted=0 ';
            // Restrict to live and current workspaces
            $this->where_hid_del .= ' AND (pages.t3ver_wsid=0 OR pages.t3ver_wsid=' . intval($this->versioningWorkspaceId) . ')';
        }
    }

	/**
	 * @param array $pagerow
	 * @param boolean $disable
	 * @return string
	 */
	public function getExtURL($pagerow, $disable = 0)
    {
		if (in_array($pagerow['doktype'], $this->externalDoktypes) && !$disable) {
			$redirectTo = $this->urltypes[$pagerow['urltype']] . $pagerow['url'];
			// If relative path, prefix Site URL:
			$uI = parse_url($redirectTo);
			// Relative path assumed now.
			if (!$uI['scheme'] && substr($redirectTo, 0, 1) != '/') {
				$redirectTo = \TYPO3\CMS\Core\Utility\GeneralUtility::getIndpEnv('TYPO3_SITE_URL') . $redirectTo;
			}

			return $redirectTo;
		} else {
            return parent::getExtURL($pagerow, $disable);
        }
	}
}